function [ t,concs ] = RevMicMen(S0,P0,KmS,KmP,Keq,VmaxS,Tf)
%RevMicMen Calculates time courses from M-M kinetic parameters
% Jason Kahn U Maryland 2/25/2016  version 1.01
%   Outputs concs = S and P concentrations as column vectors of one matrix,
%   vs. time as a column vector
%   Typical Usage: RevMicMen(1e-4,0,1e-5,1e-3,10,10e-5,2.5);
%   By Briggs-Haldane, VmaxR = (VmaxF*KmR)/(Keq*KmF) so it is not specified
%       intependently.
%   Program integrates vf = ([S] - [P]/Keq)*(Vmaxf/(KmS + [S] + [P]*KmS/KmP))
%   See http://www.mathworks.com/help/matlab/math/ordinary-differential-equations.html
%   Syntax is [t,y] = solver(odefun,tspan,y0,options)
% Sample for how to pass arguments - just nest the function
% http://www.mathworks.com/help/matlab/math/ordinary-differential-equations.html#brfharp-9
% function [t,y] = solve_vdp(mu)
% tspan = [0 max(20, 3*mu)];
% y0 = [2; 0];
% 
% % Call the ODE solver ode15s.
% [t,y] = ode15s(@vdp,tspan,y0);
% 
%     % Define the ODE function as nested function, 
%     % using the parameter mu.
%     function dydt = vdp(t,y)
%     dydt = [y(2); mu*(1-y(1)^2)*y(2)-y(1)];
%     end
% end
%tspan = [0 Tf];
% For finer gradations (presumably incrementally slower), specify time
% increments; if things change very rapidly presumably this is a bad idea.
% This is convenient also because can make multiple graphs with consistent
% t variable or collect concs in larger matrices.
tspan = 0:Tf/100:Tf;
concs0 = [S0 ; P0];
% The ode45 solver takes forever -- this is probably a "stiff" system that
% is numerically unstable. 
options = odeset('AbsTol',1e-8);
% Can use ode23 or ode15s. ode15s seems to be about 40% faster.
[t,concs] = ode15s(@Mmvel,tspan,concs0,options);

    function dconcsdt = Mmvel(~,concs)
        velo = (concs(1) - concs(2)/Keq)*(VmaxS/(KmS + concs(1) + concs(2)*KmS/KmP));
        dconcsdt = [-velo ; velo];
    end


end

